@@ -88,10 +88,36 @@ void grid_row_uri_range_put(
     struct row *row, int col, const char *uri, uint64_t id);
 void grid_row_uri_range_erase(struct row *row, int start, int end);
 
+void grid_row_underline_range_put(
+    struct row *row, int col, struct underline_range_data data);
+void grid_row_underline_range_erase(struct row *row, int start, int end);
+
 static inline void
-grid_row_uri_range_destroy(struct row_uri_range *range)
+grid_row_uri_range_destroy(struct row_range *range)
 {
-    free(range->uri);
+    free(range->uri.uri);
+}
+
+static inline void
+grid_row_underline_range_destroy(struct row_range *range)
+{
+}
+
+static inline void
+grid_row_range_destroy(struct row_range *range, enum row_range_type type)
+{
+    switch (type) {
+    case ROW_RANGE_URI: grid_row_uri_range_destroy(range); break;
+    case ROW_RANGE_UNDERLINE: grid_row_underline_range_destroy(range); break;
+    }
+}
+
+static inline void
+grid_row_ranges_destroy(struct row_ranges *ranges, enum row_range_type type)
+{
+    for (int i = 0; i < ranges->count; i++) {
+        grid_row_range_destroy(&ranges->v[i], type);
+    }
 }
 
 static inline void
@@ -102,9 +128,10 @@ grid_row_reset_extra(struct row *row)
     if (likely(extra == NULL))
         return;
 
-    for (size_t i = 0; i < extra->uri_ranges.count; i++)
-        grid_row_uri_range_destroy(&extra->uri_ranges.v[i]);
+    grid_row_ranges_destroy(&extra->uri_ranges, ROW_RANGE_URI);
+    grid_row_ranges_destroy(&extra->underline_ranges, ROW_RANGE_UNDERLINE);
     free(extra->uri_ranges.v);
+    free(extra->underline_ranges.v);
 
     free(extra);
     row->extra = NULL;
