@@ -8,7 +8,7 @@
 #include <sys/epoll.h>
 
 #define LOG_MODULE "osc"
-#define LOG_ENABLE_DBG 0
+#define LOG_ENABLE_DBG 1
 #include "log.h"
 #include "base64.h"
 #include "config.h"
@@ -64,6 +64,14 @@ osc_to_clipboard(struct terminal *term, const char *target,
         return;
     }
 
+    const bool copy_allowed = term->conf->security.osc52 == OSC52_ENABLED
+                              || term->conf->security.osc52 == OSC52_COPY_ENABLED;
+
+    if (!copy_allowed) {
+        LOG_DBG("ignoring copy request: disabled in configuration");
+        return;
+    }
+
     char *decoded = base64_decode(base64_data, NULL);
     if (decoded == NULL) {
         if (errno == EINVAL)
@@ -190,6 +198,13 @@ osc_from_clipboard(struct terminal *term, const char *source)
         return;
     }
 
+    const bool paste_allowed = term->conf->security.osc52 == OSC52_ENABLED
+                               || term->conf->security.osc52 == OSC52_PASTE_ENABLED;
+    if (!paste_allowed) {
+        LOG_DBG("ignoring paste request: disabled in configuration");
+        return;
+    }
+
     /* Use clipboard if no source has been specified */
     char src = source[0] == '\0' ? 'c' : 0;
     bool from_clipboard = src == 'c';
@@ -557,9 +572,13 @@ osc_notify(struct terminal *term, char *string)
         return;
     }
 
+    char *msgdup = NULL;
+    if (msg != NULL)
+        msgdup = xstrdup(msg);
+
     notify_notify(term, &(struct notification){
         .title = xstrdup(title),
-        .body = xstrdup(msg),
+        .body = msgdup,
         .expire_time = -1,
         .focus = true,
     });
@@ -1227,10 +1246,22 @@ osc_dispatch(struct terminal *term)
         osc_uri(term, string);
         break;
 
-    case 9:
+    case 9: {
         /* iTerm2 Growl notifications */
+        const char *sep = strchr(string, ';');
+        if (sep != NULL) {
+            errno = 0;
+            char *end = NULL;
+            strtoul(string, &end, 10);
+            if (end == sep && errno == 0) {
+                /* Ignore ConEmu/Windows Terminal escape */
+                break;
+            }
+        }
+
         osc_notify(term, string);
         break;
+    }
 
     case 10:    /* fg */
     case 11:    /* bg */
